
/*
 * name: icmp.c
 * this file is part of the "Net-Injector" project. It defines the write_icmp() function
 * Copyright (C) 2002 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * date: 04 17 2003
 * rev: 0.8

 	0 ______________________15-16________________________31
 	|       |                Message ICMP (xb)            |
	|  ID.  |____________________________________________ |
	|       |        |         |             |            |
	|       |Type(8b)| Code(8b)|Checksum(16b)|sequence(xb)|
	|_______|________|_________|_____________|____________|
*/

#include "neti.h"

/* function that create the ICMP header and data field 
the result is a pointer (*icmp_packet) to the begining of ICMP header
*/

int write_icmp(char *icmp_packet, int size){

	struct icmphdr *icmph;
	struct icmp_timestamp *tstamp;
	struct timeval tv;


	icmph =  (struct icmphdr*) (icmp_packet);    /* address of icmp-header */
	data = (char*) (icmp_packet + ICMPHDR_SIZE);	/* address of begining of payload */

	memset(data, 0, myhdr->data_size);
	memset(icmp_packet, 0, size);


	/*filling ICMP header */
	icmph->type = myhdr->icmp_type;
	icmph->code = myhdr->icmp_code;
	if ( (myhdr->icmp_type == 8) || (myhdr->icmp_type == 8)) {
		icmph->un.echo.id = htons (myhdr->icmp_echo_id);
		icmph->un.echo.sequence = htons(myhdr->icmp_echo_sequence);
	}
	if (myhdr->icmp_type == 5) {
		icmph->un.gateway = inet_addr (myhdr->icmp_redirect_gateway);
	}
	if (myhdr->icmp_type == 13) {
		tstamp = (struct icmp_timestamp*) (data);
		gettimeofday(&tv, NULL);
		tstamp->originate = htonl((tv.tv_sec % 86400) * 1000 + tv.tv_usec / 1000);
		tstamp->receive = 0;
		tstamp->transmit = 0;
	}

	icmph->checksum=0;


	icmph->checksum = sum((u_int16_t *) icmp_packet, size);

	return EXIT_SUCCESS;
}
