
/*
 * name: listen.c
 * this file is part of the "Net-Injector" project. It defines the sniff() function
 * Copyright (C) 2002 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * date: 04 17 2003
 * rev: 0.8
*/


#include "neti.h"


int sniff(int udp_sniff_sock, int tcp_sniff_sock, int icmp_sniff_sock, int keep, u_int32_t timeout, u_int16_t rlookup) {

	int recieved, fromlen;
	char buffer[8192], src_hostname[1024], dst_hostname[1024];
	struct sockaddr_in from, gateway;
	struct ip  *sniff_ip;		/* we use this version of header (struct ip) because it handles a real sockaddr_in member-struct */
	struct tcphdr *sniff_tcp;
	struct udphdr *sniff_udp;
	struct icmphdr *sniff_icmp;
	struct icmp_timestamp *tstamp;
	struct timeval tv_out;
	u_int32_t printed_packets;

	fd_set readfs;
	int maxsock;

	printed_packets = 0;
	tv_out.tv_sec = timeout;
	tv_out.tv_usec = 0;
    maxsock = max(udp_sniff_sock, tcp_sniff_sock, icmp_sniff_sock)+1;





	while(1){

		fromlen = sizeof(from);

		FD_ZERO (&readfs);
		FD_SET(udp_sniff_sock, &readfs);
		FD_SET(tcp_sniff_sock, &readfs);
		FD_SET(icmp_sniff_sock, &readfs);
		select(maxsock, &readfs, NULL, NULL, &tv_out);
		
		
		/* in case we sniffed an UDP packet */
			
		if (FD_ISSET(udp_sniff_sock, &readfs)) {
				
				recieved = recvfrom(udp_sniff_sock, buffer, sizeof buffer, 0, (struct sockaddr *)&from, &fromlen);
				sniff_ip = (struct ip *)buffer;
				
				if ( !(waitfor->filter_on_udp + waitfor->filter_on_tcp + waitfor->filter_on_icmp) || (waitfor->filter_on_udp)) {
					if ((! waitfor->filter_on_target_addr) || (sniff_ip->ip_src.s_addr == myhdr->inet_dst_ip)) {
						
						if (rlookup){
							reverse_lookup(src_hostname, inet_ntoa(sniff_ip->ip_src));
							reverse_lookup(dst_hostname, inet_ntoa(sniff_ip->ip_dst)); 
							
						}else{
							strncpy(src_hostname, inet_ntoa(sniff_ip->ip_src), 1024);
							strncpy(dst_hostname, inet_ntoa(sniff_ip->ip_dst), 1024);
						}
						
						sniff_udp = (struct udphdr *)(buffer + IPHDR_SIZE);
						printed_packets++;

						fprintf(stdout, "Received ");
						fprintf(stdout, "<UDP>");
						fprintf(stdout, "-[len %d]", recieved);
						fprintf(stdout, "-%s", src_hostname);
						fprintf(stdout, "-TTL(%d)", sniff_ip->ip_ttl);
						fprintf(stdout, "-TOS(%d)", sniff_ip->ip_tos);
						fprintf(stdout, "[%d] -> ", ntohs(sniff_udp->source));
						fprintf(stdout, "%s", dst_hostname);
						fprintf(stdout, "[%d]", ntohs(sniff_udp->dest));
						fprintf(stdout, "\n");
					}
				} 
				memset(sniff_ip, 0, IPHDR_SIZE);

				if ((keep == 0) && (printed_packets != 0)) break;
				else continue;
		}

		
		
		/* in case we sniffed a TCP packet */

		if (FD_ISSET(tcp_sniff_sock, &readfs)) {
				recieved = recvfrom(tcp_sniff_sock, buffer, sizeof buffer, 0, (struct sockaddr *)&from, &fromlen);
				sniff_ip = (struct ip *)buffer;


				if ( !(waitfor->filter_on_udp + waitfor->filter_on_tcp + waitfor->filter_on_icmp) || (waitfor->filter_on_tcp)) {
					if ((! waitfor->filter_on_target_addr) || (sniff_ip->ip_src.s_addr == myhdr->inet_dst_ip)) { 

						if (rlookup){
							reverse_lookup(src_hostname, inet_ntoa(sniff_ip->ip_src));
							reverse_lookup(dst_hostname, inet_ntoa(sniff_ip->ip_dst)); 
							
						}else{
							strncpy(src_hostname, inet_ntoa(sniff_ip->ip_src), 1024);
							strncpy(dst_hostname, inet_ntoa(sniff_ip->ip_dst), 1024);
						}
						
						sniff_tcp = (struct tcphdr *)(buffer + IPHDR_SIZE);
						printed_packets++;
						fprintf(stdout, "Received ");
						fprintf(stdout, "<TCP>");
						fprintf(stdout, "-[len %d]", recieved);
						if (ntohs(sniff_tcp->syn) != 0) fprintf(stdout, "-SYN");
						if (ntohs(sniff_tcp->ack) != 0) fprintf(stdout, "-ACK");
						if (ntohs(sniff_tcp->psh) != 0) fprintf(stdout, "-PSH");
						if (ntohs(sniff_tcp->rst) != 0) fprintf(stdout, "-RST");
						if (ntohs(sniff_tcp->fin) != 0) fprintf(stdout, "-FIN");
						if (ntohs(sniff_tcp->urg) != 0) fprintf(stdout, "-URG");
						fprintf(stdout, "-WIN(%d)", sniff_tcp->window);
						fprintf(stdout, "-%s", src_hostname);
						fprintf(stdout, "[%d]", ntohs(sniff_tcp->source));
						fprintf(stdout, "-TTL(%d)", sniff_ip->ip_ttl);
						fprintf(stdout, "-TOS(%d)", sniff_ip->ip_tos);
						fprintf(stdout, " -> %s", dst_hostname);
						fprintf(stdout, "[%d]", ntohs(sniff_tcp->dest));
						fprintf(stdout, "\n");
					}
				} 
			memset(sniff_ip, 0, IPHDR_SIZE);

				if ((keep == 0) && (printed_packets != 0)) break;
				else continue;
		}


		/* in case we sniffed an ICMP packet */

		if (FD_ISSET(icmp_sniff_sock, &readfs)) {
				recieved = recvfrom(icmp_sniff_sock, buffer, sizeof buffer, 0, (struct sockaddr *)&from, &fromlen);
				sniff_ip = (struct ip *)buffer;

				if ( !(waitfor->filter_on_udp + waitfor->filter_on_tcp + waitfor->filter_on_icmp) || (waitfor->filter_on_icmp)) {
					if ((! waitfor->filter_on_target_addr) || (sniff_ip->ip_src.s_addr == myhdr->inet_dst_ip)) { 
						
						if (rlookup){
							reverse_lookup(src_hostname, inet_ntoa(sniff_ip->ip_src));
							reverse_lookup(dst_hostname, inet_ntoa(sniff_ip->ip_dst)); 
							
						}else{
							strncpy(src_hostname, inet_ntoa(sniff_ip->ip_src), 1024);
							strncpy(dst_hostname, inet_ntoa(sniff_ip->ip_dst), 1024);
						}
						
						sniff_icmp = (struct icmphdr *)(buffer + IPHDR_SIZE);
						printed_packets++;

						fprintf(stdout, "Received ");
						fprintf(stdout, "<ICMP>");
						fprintf(stdout, "-[len %d]", recieved);
						if (sniff_icmp->type==0) {
							fprintf(stdout, "-[ECHO_REPLY]");
							fprintf(stdout, "-[ID %d]", ntohs(sniff_icmp->un.echo.id));
						}
						if (sniff_icmp->type==3){
							fprintf(stdout, "-[DEST_UNREACH]");
							if (sniff_icmp->code==0) fprintf(stdout, "-[NET_UNREACH]");
							if (sniff_icmp->code==1) fprintf(stdout, "-[HOST_UNREACH]");
							if (sniff_icmp->code==2) fprintf(stdout, "-[PROT_UNREACH]");
							if (sniff_icmp->code==3) fprintf(stdout, "-[PORT_UNREACH]");
							if (sniff_icmp->code==4) fprintf(stdout, "-[FRAG_NEEDED]");
							if (sniff_icmp->code==5) fprintf(stdout, "-[SR_FAILED]");
							if (sniff_icmp->code==6) fprintf(stdout, "-[NET_UNKNOWN]");
							if (sniff_icmp->code==7) fprintf(stdout, "-[HOST_UNKNOWN]");
							if (sniff_icmp->code==8) fprintf(stdout, "-[HOST_ISOLATED]");
							if (sniff_icmp->code==9) fprintf(stdout, "-[NET_ANO]");
							if (sniff_icmp->code==10) fprintf(stdout, "-[HOST_ANO]");
							if (sniff_icmp->code==11) fprintf(stdout, "-[NET_UNR_TOS]");
							if (sniff_icmp->code==12) fprintf(stdout, "-[HOST_UNR_TOS]");
							if (sniff_icmp->code==13) fprintf(stdout, "-[PKT_FILTERED]");
							if (sniff_icmp->code==14) fprintf(stdout, "-[PREC_VIOLATION]");
							if (sniff_icmp->code==15) fprintf(stdout, "-[PREC_CUTOFF]");
						}
						if (sniff_icmp->type==4) fprintf(stdout, "-[SOURCE_QUENCH]");
						if (sniff_icmp->type==5){
							fprintf(stdout, "-[REDIRECT]");
							if (sniff_icmp->code==0) fprintf(stdout, "-[REDIR_NET]");
							if (sniff_icmp->code==1) fprintf(stdout, "-[REDIR_HOST]");
							if (sniff_icmp->code==2) fprintf(stdout, "-[REDIR_NETTOS]");
							if (sniff_icmp->code==3) fprintf(stdout, "-[REDIR_HOSTTOS]");
							gateway.sin_addr.s_addr = sniff_icmp->un.gateway;
							fprintf(stdout, "-[GW %s]", inet_ntoa(gateway.sin_addr));
						}
						if (sniff_icmp->type==8) {
							fprintf(stdout, "-[ECHO_REQUEST]");
							fprintf(stdout, "-[ID %d]", ntohs(sniff_icmp->un.echo.id));
						}
						if (sniff_icmp->type==11){
							fprintf(stdout, "-[TIME_EXCEEDED]");
							if (sniff_icmp->code==0) fprintf(stdout, "-[EXC_TTL]");
							if (sniff_icmp->code==1) fprintf(stdout, "-[EXC_FRAGTIME]");
						}
						if (sniff_icmp->type==12) fprintf(stdout, "-[PARAMETER PROB]");
						if (sniff_icmp->type==13) {
							fprintf(stdout, "-[TIMESTAMP");
							tstamp = (struct icmp_timestamp *) (buffer + IPHDR_SIZE + ICMPHDR_SIZE);
							fprintf(stdout, "-%d]", ntohl(tstamp->originate));
						}
						if (sniff_icmp->type==14) {
							fprintf(stdout, "-[TIMESTAMP REPLY");
							tstamp = (struct icmp_timestamp *) (buffer + IPHDR_SIZE + ICMPHDR_SIZE);
							fprintf(stdout, "-%d]", ntohl(tstamp->originate));
						}
						if (sniff_icmp->type==15) fprintf(stdout, "-[INFO_REQUEST]");
						if (sniff_icmp->type==16) fprintf(stdout, "-[INFO_REPLY]");
						if (sniff_icmp->type==17) fprintf(stdout, "-[ADDRESS]");
						if (sniff_icmp->type==18) fprintf(stdout, "-[ADDRESSREPLY]");
						fprintf(stdout, "-%s", src_hostname);
						fprintf(stdout, "-TTL(%d)", sniff_ip->ip_ttl);
						fprintf(stdout, "-TOS(%d)", sniff_ip->ip_tos);
						fprintf(stdout, " -> ");
						fprintf(stdout, "%s", dst_hostname);
						fprintf(stdout, "\n");
					}
				} 
			memset(sniff_ip, 0, IPHDR_SIZE);

				if ((keep == 0) && (printed_packets != 0)) break;
				else continue;
		}else{
				break;
		}

	} /* end while */

	return EXIT_SUCCESS;

} /* end sniff	*/



