/* 
 * name: parse_cfile.c
 * this file is part of the "Net-Injector" project. It defines functions used to
 * analyze the config file.
 * Copyright (C) 2002 by Jean Philippe GUILLEMIN <jp.guillemin@free.fr>
 * license: This software is under GPL license
 * date: 04 17 2003
 * rev: 0.8
*/ 

#include "neti.h"

/************************************************************************************/
/* try to find parameters in given config file using the func parse_from_config()	*/
/************************************************************************************/
int take_args_from_file(char *cfile){

	char *parameter, *char_param_value;
	u_int32_t uint_param_value;
	parameter = malloc(128); 	
	char_param_value = malloc(128); 
	
	strncpy (parameter, "interface", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		strncpy(myhdr->interface, char_param_value, 32);
	}
	
	strncpy (parameter, "source_address", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->inet_src_ip = inet_addr(char_param_value);
		strncpy(myhdr->src_ip, char_param_value, 32);
		use_this_address = 1;
	}
	
	strncpy (parameter, "source_port", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->src_port = uint_param_value;
	}
	
	strncpy (parameter, "protocol", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (!strcmp (char_param_value, "UDP")) myhdr->proto = 17;
		if (!strcmp (char_param_value, "TCP")) myhdr->proto = 6;
		if (!strcmp (char_param_value, "ICMP")) myhdr->proto = 1;	
	}
	
	strncpy (parameter, "dest_port", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->dst_port = uint_param_value;
		myhdr->inet_dst_port = htons(myhdr->dst_port);
	}
	
	strncpy (parameter, "dest_address", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->inet_dst_ip = inet_addr(char_param_value);
		strncpy(myhdr->dst_ip, char_param_value, 32);
	}
						
	strncpy (parameter, "syn", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->syn = uint_param_value;
	}
	
						
	strncpy (parameter, "psh", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->psh = uint_param_value;
	}
											
	strncpy (parameter, "ack", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->ack = uint_param_value;
	}
											
	strncpy (parameter, "rst", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->rst = uint_param_value;
	}
										
	strncpy (parameter, "fin", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->fin = uint_param_value;
	}
															
	strncpy (parameter, "urg", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->urg = uint_param_value;
	}
	
	strncpy (parameter, "tcp_timestamp", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) myhdr->tcp_opt_size = 12;
	}
	
	strncpy (parameter, "ttl", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->ttl = uint_param_value;
	}
	
	strncpy (parameter, "window", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->window = atoi(char_param_value);
	}
	
	strncpy (parameter, "icmp_type", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->icmp_type = atoi(char_param_value);
		if (myhdr->icmp_type == 13) myhdr->data_size = ICMP_TIMESTAMP_SIZE;
	}
	
	strncpy (parameter, "icmp_code", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->icmp_code = atoi(char_param_value);
	}
	
	strncpy (parameter, "icmp_redirect_gateway", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		strncpy(myhdr->icmp_redirect_gateway, char_param_value, 32);
	}
	
	strncpy (parameter, "icmp_echo_id", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->icmp_echo_id = atoi(char_param_value);
	}
	
	strncpy (parameter, "offset", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		myhdr->offset = atoi(char_param_value);
	}
	
	strncpy (parameter, "filter_on_udp", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) waitfor->filter_on_udp = 1;
	}
	
	strncpy (parameter, "filter_on_tcp", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) waitfor->filter_on_tcp = 1;
	}
	
	strncpy (parameter, "filter_on_icmp", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) waitfor->filter_on_icmp = 1;
	}
	
	strncpy (parameter, "filter_on_target_addr", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) waitfor->filter_on_target_addr = 1;
	}
	
	strncpy (parameter, "tos_minimum_cost", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) myhdr->iptos |= 0x2;
	}
	
	strncpy (parameter, "tos_maximum_reliability", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) myhdr->iptos |= 0x4;
	}
	
	strncpy (parameter, "tos_maximum_throughput", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) myhdr->iptos |= 0x8;
	}
	
	strncpy (parameter, "tos_minimum_delay", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) myhdr->iptos |= 0x10;
	}
	
	strncpy (parameter, "keep_sniffing", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) keep_sniffing = 1;
	}
	
	strncpy (parameter, "timeout", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		time_to_wait_for_packet = uint_param_value;
	}
	
	strncpy (parameter, "no_lookup", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) no_lookup = 1;
	}
	
	strncpy (parameter, "do_reverse_lookup", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) do_reverse_lookup = 1;
	}
	
	strncpy (parameter, "be_promiscuous", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) be_promiscuous = 1;
	}
	
	strncpy (parameter, "be_blind", 128);
	if (parse_from_config(&uint_param_value, char_param_value, cfile, parameter) == 0){
		if (uint_param_value == 1) be_blind = 1;
	}
return(EXIT_SUCCESS);
}

/*****************************************************************************/
/*      scan the config file and return the associated value in uint & char	 */
/*****************************************************************************/

int parse_from_config(u_int32_t *uint_value, char *char_value, char *filename, char *param ){
	int c;
	FILE *file;
	char *sep, *buffer, *line;
	line = malloc(1024); 
	buffer = malloc(1024); 
	sep = malloc(1024); 
	c = 0;	
	
	if((file=fopen(filename, "rb")) == 0) {
		fprintf (stderr,"Error opening file %s !\n",filename);
		exit(EXIT_FAILURE);
	}

	while (fgets (line, 1024, file) !=NULL) {
		if (strstr (line, "[") !=0) continue;
		if (strstr (line, "#") !=0) continue;
		c = sscanf (line, "%s %s %s", buffer, sep, char_value);
		if (strncmp (param, buffer, 128) == 0) {
			*uint_value = atoi(char_value);
			fclose (file);
			return(EXIT_SUCCESS);
		}	
	}
fclose (file);
return(EXIT_FAILURE);
}
